searchNodes=[{"ref":"Bounds.html","title":"Bounds","type":"module","doc":"Bounds is a formalization of the {pos, len} tuple that is used in Erlang to slice binaries. A Bounds value is similar to a Range value, with a few differences: A Bounds value can be zero-length. (A Range value must represent at least one element.) A Bounds value is always ascending. (A Range value may be descending.) A Bounds value always has non-negative values. (A Range value may represent negative values.) Point Bounds The relaxation of the nonzero-size rule from Range, means that Bounds values may represent single points, equivalent to {pos, 0} tuples. Such Bounds values are said to be &quot;point bounds&quot; or &quot;point-bounded.&quot; A Bounds value that is not point-bounded is said to be a &quot;range bound&quot; or &quot;range-bounded&quot;, as there exists an equivalent Range value to it. Many functions in Bounds return point bounds when used near the Bounds value&#39;s end points. For example, difference/2 subtracts an open interval from a closed interval; if used with the same argument on both sides, the mathematical result will be [a, b] - (a, b) = {(a, a), (b, b)} — a list of two point bounds representing the left-over endpoints from the closed interval. This is an implementation choice made to allow the user of Bounds to not worry about whether their Bounds values represent open or closed intervals. Rather than explicitly defining Bounds values as representing open, half-closed, or closed intervals at all times, Bounds values instead represent generic intervals, which the algorithms in Bounds interpret as open, half-closed, or closed, either because that is the only useful interpretation for the given operation, or because a particular interpretation yields a value containing the most information—information from which the results of operations on the other types of intervals can be deduced as corollaries. To return to the difference/2 example, if you want to compute the difference between two closed intervals represented as Bounds values, you can pass the results of difference/2 to the function ranges/1 to find only the range-bounded (i.e. not point-bounded) elements, or use the predicates point?/1 or range?/1 to filter for these values yourself. Implementation Bounds values are normalized as an interval [lower, upper] for convenience of calculation. All functions in the Bounds module expect a %Bounds{} to have been created by a call to a constructor function (e.g. new/2, from_poslen/1, from_range/1.) If you construct a Bounds value yourself, the following guard must hold: %Bounds{lower: lower, upper: upper} when is_integer(lower) and is_integer(upper) and lower &gt;= 0 and upper &gt;= lower Enumeration Like Range, Bounds implements Enumerable; and thus, like Range values, Bounds values can be understood as a compact representation of an equivalent list value. Unlike Range, Bounds has several decorators which implement Enumerable differently, to represent the different, equivalent abstractions that a Bounds value can be understood as. A Bounds value, passed directly to Enum functions, will enumerate as a collection of all integer points in the closed interval [lower, upper]. If your algorithm wants &quot;point&quot; Bounds values (values where lower == upper) to be included as values in the enumeration, then this is the approach you want. A common use-case is enumerating all half-closed intervals of some length n (e.g. [lower, lower + n)) which are contained by a given Bounds value. For example, if the Bounds value represents the bounds of a binary, then you might want the bounds of each byte of the binary: [0, 1), [1, 2), etc. The functions chunk_every/2, split_stepwise/2, and partitioned/3 will help with this."},{"ref":"Bounds.html#chunk_every/3","title":"Bounds.chunk_every/3","type":"function","doc":"Returns a Bounds.Stepwise decorator value, which implements Enumerable. The values enumerated from a Bounds.Stepwise decorator are themselves Bounds values, representing a set of contiguous intervals, each of size step_size. The enumeration always begins with the interval [0, step_size) (if it exists.) Any bounded interval smaller than step_size is not considered a part of the enumeration. This enumeration strategy is useful when you have a sequence of unit-sized chunks (like the bytes in a binary), in which a representation for one element is encoded as step_size contiguous chunks. The enumerated values will then represent the bounds of the representations of all potentially-valid elements. Examples Get the bounds of each single byte of a binary: iex&gt; Bounds.from_binary(&quot;foo&quot;) |&gt; Bounds.chunk_every(1) |&gt; Enum.to_list() [%Bounds{lower: 0, upper: 1}, %Bounds{lower: 1, upper: 2}, %Bounds{lower: 2, upper: 3}] Get the bounds of a sequence of 32-bit values in a binary: iex&gt; Bounds.from_binary(&quot;0123456789&quot;) |&gt; Bounds.chunk_every(4, partials: :discard) |&gt; Enum.to_list() [%Bounds{lower: 0, upper: 4}, %Bounds{lower: 4, upper: 8}]"},{"ref":"Bounds.html#clamp/2","title":"Bounds.clamp/2","type":"function","doc":"Creates a new Bounds value by limiting the lower and upper bounds of value_bounds to be no more than the lower and upper bounds of clamp_bounds. If value_bounds fully contains clamp_bounds, the result of the clamp will be clamp_bounds exactly."},{"ref":"Bounds.html#contiguous?/2","title":"Bounds.contiguous?/2","type":"function","doc":"Determines whether the Bounds values a and b (both interpreted as half-closed intervals) can be joined to form a single larger interval, with no discontinuities or overlap. See also: overlap?/2, disjoint?/2, subset?/2 Examples Disjoint intervals fail: iex&gt; Bounds.contiguous?(Bounds.new(0, 4), Bounds.new(5, 5)) false Overlapping intervals fail: iex&gt; Bounds.contiguous?(Bounds.new(0, 6), Bounds.new(5, 5)) false Contiguous intervals succeed: iex&gt; Bounds.contiguous?(Bounds.new(0, 5), Bounds.new(5, 5)) true"},{"ref":"Bounds.html#difference/2","title":"Bounds.difference/2","type":"function","doc":"Removes the open interval sub_bounds from the closed interval bounds and returns whatever is left over. If sub_bounds has one or both endpoints in common with bounds, point (zero-length) bounds will be left over at the shared endpoint. Examples Subtracting from the middle produces two interval-bounds: iex&gt; Bounds.difference(%Bounds{lower: 0, upper: 10}, %Bounds{lower: 3, upper: 6}) [%Bounds{lower: 0, upper: 3}, %Bounds{lower: 6, upper: 10}] Subtracting from one end produces one point-bound and one interval-bound: iex&gt; Bounds.difference(%Bounds{lower: 0, upper: 10}, %Bounds{lower: 0, upper: 2}) [%Bounds{lower: 0, upper: 0}, %Bounds{lower: 2, upper: 10}] Subtracting everything produces two point-bounds: iex&gt; Bounds.difference(%Bounds{lower: 0, upper: 10}, %Bounds{lower: 0, upper: 10}) [%Bounds{lower: 0, upper: 0}, %Bounds{lower: 10, upper: 10}] Subtracting from a point always produces the same point: iex&gt; Bounds.difference(%Bounds{lower: 5, upper: 5}, %Bounds{lower: 0, upper: 10}) [%Bounds{lower: 5, upper: 5}]"},{"ref":"Bounds.html#disjoint?/2","title":"Bounds.disjoint?/2","type":"function","doc":"Determines whether the Bounds values a and b have no points in common when both are interpreted as half-closed intervals. See also: overlap?/2 Examples Disjoint values succeed: iex&gt; Bounds.disjoint?(Bounds.from_range(1..5), Bounds.from_range(4..10)) false iex&gt; Bounds.disjoint?(Bounds.from_range(1..5), Bounds.from_range(5..10)) false Everything else fails: iex&gt; Bounds.disjoint?(Bounds.from_range(1..5), Bounds.from_range(6..10)) true iex&gt; Bounds.disjoint?(Bounds.from_range(1..10), Bounds.from_range(1..10)) false iex&gt; Bounds.disjoint?(Bounds.from_range(1..10), Bounds.from_range(3..6)) false"},{"ref":"Bounds.html#endpoints/1","title":"Bounds.endpoints/1","type":"function","doc":"Returns the end points (i.e. the points on either end) of the closed interval represented by the Bounds value. If the Bounds value represents a point, there will be one end point returned (the same point.) Otherwise, there will be two points returned."},{"ref":"Bounds.html#from_binary/1","title":"Bounds.from_binary/1","type":"function","doc":"Constructs a Bounds value representing the bounds of the given binary bin, i.e. the interval: [0, byte_size(bin))"},{"ref":"Bounds.html#from_integer/1","title":"Bounds.from_integer/1","type":"function","doc":"Creates a point-bounded Bounds value equivalent to the integer offset point."},{"ref":"Bounds.html#from_poslen/1","title":"Bounds.from_poslen/1","type":"function","doc":"Converts a {pos, len} tuple to an equivalent Bounds value."},{"ref":"Bounds.html#from_range/1","title":"Bounds.from_range/1","type":"function","doc":"Converts a Range to an equivalent Bounds value."},{"ref":"Bounds.html#integers/1","title":"Bounds.integers/1","type":"function","doc":"Filters the passed Map or Enumerable value for only the Bounds values which are point-bounded, additionally casting them to integer offsets."},{"ref":"Bounds.html#join/1","title":"Bounds.join/1","type":"function","doc":"Merges an enumerable of contiguous intervals (order ignored) defined by bounds_enum into a single interval containing the superset of the points of all the intervals provided. Raises a Bounds.DisjointError exception if there would be any discontinuities in the resulting interval. See also: join/2, contiguous?/2"},{"ref":"Bounds.html#join/2","title":"Bounds.join/2","type":"function","doc":"Merges the contiguous intervals a and b into a single interval containing the superset of the points of both intervals. Raises a Bounds.DisjointError exception if a and b are not contiguous intervals. See also: contiguous?/2"},{"ref":"Bounds.html#new/1","title":"Bounds.new/1","type":"function","doc":"Casts a compatible value to a Bounds value."},{"ref":"Bounds.html#new/2","title":"Bounds.new/2","type":"function","doc":"Constructs a new Bounds value from a position and a length."},{"ref":"Bounds.html#overlap?/2","title":"Bounds.overlap?/2","type":"function","doc":"Determines whether the Bounds values a and b, have any point in common when both are interpreted as half-closed intervals. See also: disjoint?/2 Examples Disjoint values fail: iex&gt; Bounds.overlap?(Bounds.from_range(1..5), Bounds.from_range(4..10)) true iex&gt; Bounds.overlap?(Bounds.from_range(1..5), Bounds.from_range(5..10)) true Everything else succeeds: iex&gt; Bounds.overlap?(Bounds.from_range(1..5), Bounds.from_range(6..10)) false iex&gt; Bounds.overlap?(Bounds.from_range(1..10), Bounds.from_range(1..10)) true iex&gt; Bounds.overlap?(Bounds.from_range(1..10), Bounds.from_range(3..6)) true"},{"ref":"Bounds.html#partitioned/3","title":"Bounds.partitioned/3","type":"function","doc":"Returns a Bounds.Partitioned decorator value, which implements Enumerable. The values enumerated from a Bounds.Partitioned decorator will be Bounds values representing a set of contiguous intervals. Most of these will be steps of size step. The first full interval (if it exists) will be [offset, offset + step). Unlike with intervals/2, the values enumerated from a Bounds.Partitioned value will include intervals smaller than step—namely: if offset is nonzero, an initial interval [0, offset) will appear at the beginning of the enumeration (if it exists.) if, after removing the initial interval, step does not evenly divide the bounds, then a final interval [step * n, step * n + remainder) will appear at the end of the enumeration (if it exists.) This enumeration strategy is useful when you are using Bounds to compactly represent a set of values, and you wish to &quot;bin&quot; these values into contiguous bins of size step."},{"ref":"Bounds.html#point?/1","title":"Bounds.point?/1","type":"function","doc":"Determines whether a Bounds value is zero-length."},{"ref":"Bounds.html#points/1","title":"Bounds.points/1","type":"function","doc":"Filters the passed Map or Enumerable value for only the Bounds values which are point-bounded."},{"ref":"Bounds.html#range?/1","title":"Bounds.range?/1","type":"function","doc":"Determines whether a Bounds value is range-bounded — i.e. whether it has a nonzero size/1."},{"ref":"Bounds.html#ranges/1","title":"Bounds.ranges/1","type":"function","doc":"Filters the passed Map or Enumerable value for only range-bounded Bounds values."},{"ref":"Bounds.html#size/1","title":"Bounds.size/1","type":"function","doc":"Returns the size of the Bounds value when interpreted as a half-closed interval [lower, upper)."},{"ref":"Bounds.html#slice/2","title":"Bounds.slice/2","type":"function","doc":"Takes a sub-interval of bounds as a new Bounds value. slicing_bounds does not represent an absolute interval to intersect with bounds, but rather is taken as a relative offset and length, as in Enum.slice/3. This operation can be understood as having equivalent semantics to that of Enum.slice/3. The offset and length will be &quot;clamped&quot; so that the result will be fully contained by bounds."},{"ref":"Bounds.html#split_at/2","title":"Bounds.split_at/2","type":"function","doc":"Returns two sub-intervals representing the result of &quot;breaking&quot; bounds at a given position. The second argument can take a few forms, each with a different result: If the second argument is an integer ≥ 0, it will be used to calculate a point to break at, relative to bounds.lower; If the second argument is an integer &lt; 0, it will be used to calculate a point to break at, relative to bounds.upper; If the second argument is a Bounds value and Bounds.point?(bounds) is true, it will be used as an absolute point to break at. Examples Non-negative relative offsets: iex&gt; Bounds.split_at(%Bounds{lower: 5, upper: 10}, 1) [%Bounds{lower: 5, upper: 6}, %Bounds{lower: 6, upper: 10}] Negative relative offsets: iex&gt; Bounds.split_at(%Bounds{lower: 5, upper: 10}, -1) [%Bounds{lower: 5, upper: 9}, %Bounds{lower: 9, upper: 10}] Absolute break positions: iex&gt; Bounds.split_at(%Bounds{lower: 5, upper: 10}, %Bounds{lower: 7, upper: 7}) [%Bounds{lower: 5, upper: 7}, %Bounds{lower: 7, upper: 10}] If the interval is broken at one of its endpoints, one of the resulting sub-intervals will be a point bound: iex&gt; Bounds.split_at(%Bounds{lower: 0, upper: 10}, 0) [%Bounds{lower: 0, upper: 0}, %Bounds{lower: 0, upper: 10}] If a relative offset is past the end of the original bounds (in either direction), the original bounds will be returned: iex&gt; Bounds.split_at(%Bounds{lower: 0, upper: 10}, 20) [%Bounds{lower: 0, upper: 10}] iex&gt; Bounds.split_at(%Bounds{lower: 0, upper: 10}, -20) [%Bounds{lower: 0, upper: 10}]"},{"ref":"Bounds.html#split_stepwise/2","title":"Bounds.split_stepwise/2","type":"function","doc":"Splits a Bounds value into three parts, returned as a map with the following keys: :whole: the bounds of the contiguous set of values whose bounds are both 1. within the original bounds, and 2. divisible by step_size. This is equivalent to the concat/2enation of the chunk_every/2 enumeration of the given bounds at the given step_size. :partial_before: the interval extending from the beginning of the original bounds, to the beginning of whole. :partial_after: the interval extending from the end of whole, to the end of the original bounds. Any/all of the values in this map may turn out to be zero-sized &quot;point&quot; Bounds."},{"ref":"Bounds.html#strict_subset?/2","title":"Bounds.strict_subset?/2","type":"function","doc":"Determines whether the endpoints of the Bounds value part are entirely contained within the endpoints of the Bounds value whole, when both are interpreted as closed intervals. Has the additional constraint that the bounds must not be equal. See also: subset?/2 Examples Disjoint values fail: iex&gt; Bounds.strict_subset?(Bounds.from_range(1..5), Bounds.from_range(4..10)) false iex&gt; Bounds.strict_subset?(Bounds.from_range(1..5), Bounds.from_range(5..10)) false Merely overlapping values fail: iex&gt; Bounds.strict_subset?(Bounds.from_range(1..5), Bounds.from_range(6..10)) false Strict subsets succeed: iex&gt; Bounds.strict_subset?(Bounds.from_range(1..10), Bounds.from_range(3..6)) true But non-strict subsets fail: iex&gt; Bounds.strict_subset?(Bounds.from_range(1..10), Bounds.from_range(1..10)) false"},{"ref":"Bounds.html#subset?/2","title":"Bounds.subset?/2","type":"function","doc":"Determines whether the endpoints of the Bounds value part are entirely contained within the endpoints of the Bounds value whole, when both are interpreted as closed intervals. See also: strict_subset?/2 Examples Disjoint values fail: iex&gt; Bounds.subset?(Bounds.from_range(1..5), Bounds.from_range(4..10)) false iex&gt; Bounds.subset?(Bounds.from_range(1..5), Bounds.from_range(5..10)) false Merely overlapping values fail: iex&gt; Bounds.subset?(Bounds.from_range(1..5), Bounds.from_range(6..10)) false Strict subsets succeed: iex&gt; Bounds.subset?(Bounds.from_range(1..10), Bounds.from_range(3..6)) true As do non-strict subsets: iex&gt; Bounds.subset?(Bounds.from_range(1..10), Bounds.from_range(1..10)) true"},{"ref":"Bounds.html#to_integer/1","title":"Bounds.to_integer/1","type":"function","doc":"Given a point-bounded Bounds value, returns the integer offset equivalent to it. Raises an exception if the given Bounds value is not point-bounded."},{"ref":"Bounds.html#to_poslen/1","title":"Bounds.to_poslen/1","type":"function","doc":"Converts a Bounds value to an equivalent {pos, len} tuple."},{"ref":"Bounds.html#to_range/1","title":"Bounds.to_range/1","type":"function","doc":"Given a range-bounded Bounds value, returns the Range value equivalent to it. Raises an exception if the given Bounds value is not range-bounded."},{"ref":"Bounds.html#translate/2","title":"Bounds.translate/2","type":"function","doc":"Returns a new Bounds value representing the result of translating (sliding) the endpoints of bounds up or down by the integer displacement disp."},{"ref":"Bounds.DisjointError.html","title":"Bounds.DisjointError","type":"exception","doc":""},{"ref":"Bounds.Map.html","title":"Bounds.Map","type":"module","doc":""},{"ref":"Bounds.Map.html#all_at/2","title":"Bounds.Map.all_at/2","type":"function","doc":""},{"ref":"Bounds.Map.html#all_overlapping/2","title":"Bounds.Map.all_overlapping/2","type":"function","doc":""},{"ref":"Bounds.Map.html#delete/2","title":"Bounds.Map.delete/2","type":"function","doc":""},{"ref":"Bounds.Map.html#edges/1","title":"Bounds.Map.edges/1","type":"function","doc":""},{"ref":"Bounds.Map.html#fetch/2","title":"Bounds.Map.fetch/2","type":"function","doc":""},{"ref":"Bounds.Map.html#fetch!/2","title":"Bounds.Map.fetch!/2","type":"function","doc":""},{"ref":"Bounds.Map.html#get/3","title":"Bounds.Map.get/3","type":"function","doc":""},{"ref":"Bounds.Map.html#insert/2","title":"Bounds.Map.insert/2","type":"function","doc":""},{"ref":"Bounds.Map.html#keys/1","title":"Bounds.Map.keys/1","type":"function","doc":""},{"ref":"Bounds.Map.html#new/0","title":"Bounds.Map.new/0","type":"function","doc":""},{"ref":"Bounds.Map.html#put/3","title":"Bounds.Map.put/3","type":"function","doc":""},{"ref":"Bounds.Map.html#values/1","title":"Bounds.Map.values/1","type":"function","doc":""},{"ref":"Bounds.Map.html#versions_at/2","title":"Bounds.Map.versions_at/2","type":"function","doc":""},{"ref":"Bounds.Map.Records.html","title":"Bounds.Map.Records","type":"module","doc":""},{"ref":"Bounds.Map.Records.html#interval/1","title":"Bounds.Map.Records.interval/1","type":"macro","doc":""},{"ref":"Bounds.Map.Records.html#interval/2","title":"Bounds.Map.Records.interval/2","type":"macro","doc":""},{"ref":"Bounds.Map.Records.html#tree_node/1","title":"Bounds.Map.Records.tree_node/1","type":"macro","doc":""},{"ref":"Bounds.Map.Records.html#tree_node/2","title":"Bounds.Map.Records.tree_node/2","type":"macro","doc":""},{"ref":"Bounds.Partitioned.html","title":"Bounds.Partitioned","type":"module","doc":""},{"ref":"Bounds.Slice.html","title":"Bounds.Slice","type":"module","doc":""},{"ref":"Bounds.Slice.html#from_binary/1","title":"Bounds.Slice.from_binary/1","type":"function","doc":""},{"ref":"Bounds.Slice.html#from_enum/2","title":"Bounds.Slice.from_enum/2","type":"function","doc":""},{"ref":"Bounds.Slice.html#from_stream/1","title":"Bounds.Slice.from_stream/1","type":"function","doc":""},{"ref":"Bounds.Slice.html#slice/2","title":"Bounds.Slice.slice/2","type":"function","doc":"See Bounds.Sliced.slice/2."},{"ref":"Bounds.Slice.html#value/1","title":"Bounds.Slice.value/1","type":"function","doc":"See Bounds.Sliced.value/1."},{"ref":"Bounds.SlicedBinary.html","title":"Bounds.SlicedBinary","type":"module","doc":""},{"ref":"Bounds.SlicedBinary.html#slice/2","title":"Bounds.SlicedBinary.slice/2","type":"function","doc":""},{"ref":"Bounds.SlicedBinary.html#to_binary/1","title":"Bounds.SlicedBinary.to_binary/1","type":"function","doc":""},{"ref":"Bounds.SlicedEnumerable.html","title":"Bounds.SlicedEnumerable","type":"module","doc":""},{"ref":"Bounds.SlicedEnumerable.html#slice/2","title":"Bounds.SlicedEnumerable.slice/2","type":"function","doc":""},{"ref":"Bounds.SlicedEnumerable.html#to_list/1","title":"Bounds.SlicedEnumerable.to_list/1","type":"function","doc":""}]